package com.aselimov.app;

import static org.kohsuke.args4j.ExampleMode.ALL;

import java.io.IOException;

import org.kohsuke.args4j.Argument;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;

import com.aselimov.app.socket.SocketListener;
import com.aselimov.app.steady_state_analysis.LinearRegression;
import com.aselimov.app.steady_state_analysis.SteadyStateCalculator;
import com.aselimov.app.timedata.TimeData;

public class SteadyDetect {
    /**
     * Main function accepts 1 argument which is the port that should be bound.
     *
     */
    public static void main(String[] args) throws IOException {
        new SteadyDetect().doMain(args);
    }

    private SocketListener socketListener;
    private TimeData data = new TimeData();

    private SteadyStateCalculator steadyStateCalc;

    @Argument
    private Integer port = null;

    @Argument
    private Double slopeTol = null;

    @Argument
    private Integer numNeededMatching = null;

    @Argument
    private Integer windowSize = null;

    public void doMain(String[] args) throws IOException {

        CmdLineParser parser = new CmdLineParser(this);
        parser.setUsageWidth(80);
        try {
            parser.parseArgument(args);

            if (port == null)
                throw new CmdLineException(parser, "No argument is given");

        } catch (CmdLineException e) {
            // if there's a problem in the command line,
            // you'll get this exception. this will report
            // an error message.
            System.err.println(e.getMessage());
            System.err.println("java SteadyDetect [options...] port");
            // print the list of available options
            parser.printUsage(System.err);
            System.err.println();

            // print option sample. This is useful some time
            System.err.println("  Example: java SteadyDetect" + parser.printExample(ALL));

            return;
        }

        System.out.println("Binding SteadyDetect to port " + port);
        socketListener = new SocketListener(port, TimeData.TimeDataPoint.class, data);
        socketListener.start();

        // Initialize the SteadyStateCalc to the LinearRegression calculator
        steadyStateCalc = new LinearRegression(slopeTol, numNeededMatching, windowSize);
        while (true) {
            steadyStateCalc.printIfSteady(data);
        }
    }
}
